'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = Hybrid;

var _asyncToGenerator = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator2 = _interopRequireDefault(_asyncToGenerator);

var _lodash = require('lodash');

var _testRunTracker = require('./test-run-tracker');

var _testRunTracker2 = _interopRequireDefault(_testRunTracker);

var _testRun = require('../../../test-run');

var _testRun2 = _interopRequireDefault(_testRun);

var _esNext = require('../../../compiler/es-next');

var _commands = require('../../../test-run/commands');

var _runtime = require('../../../errors/runtime');

var _message = require('../../../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _getCallsite = require('../../../errors/get-callsite');

var _getCallsite2 = _interopRequireDefault(_getCallsite);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ASYNC_TO_GEN_CODE = (0, _asyncToGenerator2.default)(_lodash.noop).toString();
var REGENERATOR_FOOTPRINTS_RE = /(_regenerator(\d+).default|regeneratorRuntime).wrap\(function _callee\$\(_context\)/;

function resolveContextTestRun() {
    var testRunId = _testRunTracker2.default.getContextTestRunId();
    var testRun = _testRun2.default.activeTestRuns[testRunId];

    if (!testRun) throw new _runtime.APIError('hybridFunction', null, _message2.default.hybridFunctionCantResolveTestRun);

    return testRun;
}

function createHybridFunction(fnCode, boundTestRun) {
    var hybridFn = function hybridFunction() {
        var testRun = boundTestRun || resolveContextTestRun(boundTestRun);
        var args = [];

        // OPTIMIZATION: don't leak `arguments` object.
        for (var i = 0; i < arguments.length; i++) {
            args.push(arguments[i]);
        }var callsite = (0, _getCallsite2.default)('hybridFunction');
        var command = new _commands.ExecuteHybridFunctionCommand(fnCode, args);

        return testRun.executeCommand(command, callsite).then(function (commandResult) {
            return commandResult.fnResult;
        });
    };

    hybridFn.bindTestRun = function bindTestRun(t) {
        // NOTE: we can't use strict `t instanceof TestController`
        // check due to module circular reference
        if (!t || !(t.testRun instanceof _testRun2.default)) throw new _runtime.APIError('bindTestRun', null, _message2.default.invalidHybridTestRunBinding);

        return createHybridFunction(fnCode, t.testRun);
    };

    return hybridFn;
}

// NOTE: we use runtime APIError in most places because these errors may appear
// at the test compilation time when we don't have any test runs yet.
function Hybrid(fn) {
    var fnType = typeof fn === 'undefined' ? 'undefined' : (0, _typeof3.default)(fn);
    var calledWithNew = this instanceof Hybrid;
    var fnName = calledWithNew ? 'constructor' : 'Hybrid';
    var typeName = calledWithNew ? 'Hybrid' : null;

    if (fnType !== 'function') throw new _runtime.APIError(fnName, typeName, _message2.default.clientCodeIsNotAFunction, fnType);

    var fnCode = fn.toString();

    if (fnCode === ASYNC_TO_GEN_CODE) throw new _runtime.APIError(fnName, typeName, _message2.default.regeneratorInClientCode);

    fnCode = (0, _esNext.compileHybridFunction)(fnCode);

    // NOTE: check compiled code for regenerator injection: we have either generator
    // recompiled in Node.js 4+ for client or async function declared in function code.
    if (REGENERATOR_FOOTPRINTS_RE.test(fnCode)) throw new _runtime.APIError(fnName, typeName, _message2.default.regeneratorInClientCode);

    return createHybridFunction(fnCode, null);
}
module.exports = exports['default'];