'use strict';

exports.__esModule = true;

var _TYPE$actionIntegerOp;

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _lodash = require('lodash');

var _type = require('./type');

var _type2 = _interopRequireDefault(_type);

var _state = require('../../test-run/state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function markup(err, msgMarkup) {
    var prefix = (0, _dedent2.default)('\n        <span class="user-agent">' + err.userAgent + '</span>\n        <span class="category">' + err.category + '</span>\n    ');

    if (err.testRunState === _state2.default.inBeforeEach) prefix += '<strong>- Error in <code>beforeEach</code> hook -</strong>\n';else if (err.testRunState === _state2.default.inAfterEach) prefix += '<strong>- Error in <code>afterEach</code> hook -</strong>\n';

    msgMarkup = prefix + (0, _dedent2.default)(msgMarkup);

    if (err.screenshotPath) msgMarkup += '\n\n<div class="screenshot-info"><strong>Screenshot:</strong> <a class="screenshot-path">' + (0, _lodash.escape)(err.screenshotPath) + '</a></div>';

    return msgMarkup;
}

exports.default = (_TYPE$actionIntegerOp = {}, _TYPE$actionIntegerOp[_type2.default.actionIntegerOptionError] = function (err) {
    return markup(err, '\n        Action option <code>' + err.optionName + '</code> is expected to be an integer, but it was <code>' + err.actualValue + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionPositiveIntegerOptionError] = function (err) {
    return markup(err, '\n        Action option <code>' + err.optionName + '</code> is expected to be a positive integer, but it was <code>' + err.actualValue + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionBooleanOptionError] = function (err) {
    return markup(err, '\n        Action option <code>' + err.optionName + '</code> is expected to be a boolean value, but it was <code>' + err.actualValue + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.uncaughtErrorOnPage] = function (err) {
    return markup(err, '\n        Error on page <a href="' + err.pageDestUrl + '">' + err.pageDestUrl + '</a>:\n\n        <code>' + (0, _lodash.escape)(err.errMsg) + '</code>\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.uncaughtErrorInTestCode] = function (err) {
    return markup(err, '\n        <code>' + (0, _lodash.escape)(err.errMsg) + '</code>\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.uncaughtNonErrorObjectInTestCode] = function (err) {
    return markup(err, '\n        Uncaught ' + err.objType + ' "' + (0, _lodash.escape)(err.objStr) + '" was thrown. Throw <code>Error</code> instead.\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionSelectorTypeError] = function (err) {
    return markup(err, '\n        The selector is expected to be a string, but it was <code>' + err.actualType + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionOptionsTypeError] = function (err) {
    return markup(err, '\n        Action options is expected to be an object, null or undefined but it was <code>' + err.actualType + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.dragDestinationSelectorTypeError] = function (err) {
    return markup(err, '\n        The destination selector is expected to be a string, but it was <code>' + err.actualType + '</code>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionStringArgumentError] = function (err) {
    return markup(err, '\n        The <code>' + err.argumentName + '</code> argument is expected to be a non-empty string, but it was ' + (err.actualType !== 'empty' ? '<code>' + err.actualType + '</code>' : err.actualType) + '.\n\n         ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionElementNotFoundError] = function (err) {
    return markup(err, '\n        The specified selector does not match any element in the DOM tree.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.actionElementIsInvisibleError] = function (err) {
    return markup(err, '\n        The element that matches the specified selector is not visible.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.dragDestinationNotFoundError] = function (err) {
    return markup(err, '\n        The specified destination selector does not match any element in the DOM tree.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.dragDestinationIsInvisibleError] = function (err) {
    return markup(err, '\n        The element that matches the specified destination selector is not visible.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.missingAwaitError] = function (err) {
    return markup(err, '\n        A call to an async function is not awaited. Use the <code>await</code> keyword before actions, assertions or chains of them to ensure that they run in the right sequence.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionIntegerOp[_type2.default.externalAssertionLibraryError] = function (err) {
    return markup(err, '\n        ' + (0, _lodash.escape)(err.errMsg) + ' ' + err.getCallsiteMarkup({ stackOnly: true }) + '\n    ');
}, _TYPE$actionIntegerOp);
module.exports = exports['default'];