'use strict';

exports.__esModule = true;
exports.TestDoneCommand = exports.ExecuteHybridFunctionCommand = exports.DragToElementCommand = exports.DragCommand = exports.TypeTextCommand = exports.HoverCommand = exports.DoubleClickCommand = exports.RightClickCommand = exports.ClickCommand = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.createCommandFromObject = createCommandFromObject;
exports.isTestDoneCommand = isTestDoneCommand;
exports.isCommandRejectableByPageError = isCommandRejectableByPageError;

var _type = require('./type');

var _type2 = _interopRequireDefault(_type);

var _assignable = require('../../utils/assignable');

var _assignable2 = _interopRequireDefault(_assignable);

var _testRun = require('../../errors/test-run');

var _options = require('./options');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var EMPTY_STRING_MESSAGE = 'empty';

// Validators
function selector(option, val) {
    var type = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (type !== 'string') throw new _testRun.ActionSelectorTypeError(type);
}

function dragDestinationSelector(option, val) {
    var type = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (type !== 'string') throw new _testRun.DragDestinationSelectorTypeError(type);
}

function actionOptions(option, val) {
    var type = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (type !== 'object' && val !== null && val !== void 0) throw new _testRun.ActionOptionsTypeError(type);
}

function integer(option, val) {
    var valType = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (valType !== 'number') throw new _testRun.ActionIntegerOptionError(option, valType);

    var isInteger = !isNaN(val) && isFinite(val) && val === Math.floor(val);

    if (!isInteger) throw new _testRun.ActionIntegerOptionError(option, val);
}

function nonEmptyStringArgument(option, val) {
    var type = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (type !== 'string') throw new _testRun.ActionStringArgumentError(option, type);

    if (!val.length) throw new _testRun.ActionStringArgumentError(option, EMPTY_STRING_MESSAGE);
}

// Initializers
function initSelector(val) {
    return '(function () { return document.querySelector(\'' + val + '\') })()';
}

function initClickOptions(val) {
    return new _options.ClickOptions(val, true);
}

function initMouseOptions(val) {
    return new _options.MouseOptions(val, true);
}

function initTypeOptions(val) {
    return new _options.TypeOptions(val, true);
}

// Commands

var ClickCommand = exports.ClickCommand = function (_Assignable) {
    (0, _inherits3.default)(ClickCommand, _Assignable);

    function ClickCommand(obj) {
        (0, _classCallCheck3.default)(this, ClickCommand);

        var _this = (0, _possibleConstructorReturn3.default)(this, _Assignable.call(this, obj));

        _this.type = _type2.default.click;
        _this.selector = null;
        _this.options = null;

        _this._assignFrom(obj, true);
        return _this;
    }

    ClickCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'options', type: actionOptions, init: initClickOptions, required: true }];
    };

    return ClickCommand;
}(_assignable2.default);

var RightClickCommand = exports.RightClickCommand = function (_Assignable2) {
    (0, _inherits3.default)(RightClickCommand, _Assignable2);

    function RightClickCommand(obj) {
        (0, _classCallCheck3.default)(this, RightClickCommand);

        var _this2 = (0, _possibleConstructorReturn3.default)(this, _Assignable2.call(this, obj));

        _this2.type = _type2.default.rightClick;
        _this2.selector = null;
        _this2.options = null;

        _this2._assignFrom(obj, true);
        return _this2;
    }

    RightClickCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'options', type: actionOptions, init: initClickOptions, required: true }];
    };

    return RightClickCommand;
}(_assignable2.default);

var DoubleClickCommand = exports.DoubleClickCommand = function (_Assignable3) {
    (0, _inherits3.default)(DoubleClickCommand, _Assignable3);

    function DoubleClickCommand(obj) {
        (0, _classCallCheck3.default)(this, DoubleClickCommand);

        var _this3 = (0, _possibleConstructorReturn3.default)(this, _Assignable3.call(this, obj));

        _this3.type = _type2.default.doubleClick;
        _this3.selector = null;
        _this3.options = null;

        _this3._assignFrom(obj, true);
        return _this3;
    }

    DoubleClickCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'options', type: actionOptions, init: initClickOptions, required: true }];
    };

    return DoubleClickCommand;
}(_assignable2.default);

var HoverCommand = exports.HoverCommand = function (_Assignable4) {
    (0, _inherits3.default)(HoverCommand, _Assignable4);

    function HoverCommand(obj) {
        (0, _classCallCheck3.default)(this, HoverCommand);

        var _this4 = (0, _possibleConstructorReturn3.default)(this, _Assignable4.call(this, obj));

        _this4.type = _type2.default.hover;
        _this4.selector = null;
        _this4.options = null;

        _this4._assignFrom(obj, true);
        return _this4;
    }

    HoverCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'options', type: actionOptions, init: initMouseOptions, required: true }];
    };

    return HoverCommand;
}(_assignable2.default);

var TypeTextCommand = exports.TypeTextCommand = function (_Assignable5) {
    (0, _inherits3.default)(TypeTextCommand, _Assignable5);

    function TypeTextCommand(obj) {
        (0, _classCallCheck3.default)(this, TypeTextCommand);

        var _this5 = (0, _possibleConstructorReturn3.default)(this, _Assignable5.call(this, obj));

        _this5.type = _type2.default.typeText;
        _this5.selector = null;
        _this5.text = null;
        _this5.options = null;

        _this5._assignFrom(obj, true);
        return _this5;
    }

    TypeTextCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'text', type: nonEmptyStringArgument, required: true }, { name: 'options', type: actionOptions, init: initTypeOptions, required: true }];
    };

    return TypeTextCommand;
}(_assignable2.default);

var DragCommand = exports.DragCommand = function (_Assignable6) {
    (0, _inherits3.default)(DragCommand, _Assignable6);

    function DragCommand(obj) {
        (0, _classCallCheck3.default)(this, DragCommand);

        var _this6 = (0, _possibleConstructorReturn3.default)(this, _Assignable6.call(this, obj));

        _this6.type = _type2.default.drag;
        _this6.selector = null;
        _this6.dragOffsetX = null;
        _this6.dragOffsetY = null;
        _this6.options = null;

        _this6._assignFrom(obj, true);
        return _this6;
    }

    DragCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'dragOffsetX', type: integer, required: true }, { name: 'dragOffsetY', type: integer, required: true }, { name: 'options', type: actionOptions, init: initMouseOptions, required: true }];
    };

    return DragCommand;
}(_assignable2.default);

var DragToElementCommand = exports.DragToElementCommand = function (_Assignable7) {
    (0, _inherits3.default)(DragToElementCommand, _Assignable7);

    function DragToElementCommand(obj) {
        (0, _classCallCheck3.default)(this, DragToElementCommand);

        var _this7 = (0, _possibleConstructorReturn3.default)(this, _Assignable7.call(this, obj));

        _this7.type = _type2.default.dragToElement;

        _this7.selector = null;
        _this7.destinationSelector = null;
        _this7.options = null;

        _this7._assignFrom(obj, true);
        return _this7;
    }

    DragToElementCommand.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'selector', type: selector, init: initSelector, required: true }, { name: 'destinationSelector', type: dragDestinationSelector, init: initSelector, required: true }, { name: 'options', type: actionOptions, init: initMouseOptions, required: true }];
    };

    return DragToElementCommand;
}(_assignable2.default);

var ExecuteHybridFunctionCommand = exports.ExecuteHybridFunctionCommand = function ExecuteHybridFunctionCommand(fnCode, args) {
    (0, _classCallCheck3.default)(this, ExecuteHybridFunctionCommand);

    this.type = _type2.default.execHybridFn;
    this.fnCode = fnCode;
    this.args = args;
};

var TestDoneCommand = exports.TestDoneCommand = function TestDoneCommand() {
    (0, _classCallCheck3.default)(this, TestDoneCommand);

    this.type = _type2.default.testDone;
};

// Factory


function createCommandFromObject(obj) {
    if (obj.type === _type2.default.click) return new ClickCommand(obj);

    if (obj.type === _type2.default.rightClick) return new RightClickCommand(obj);

    if (obj.type === _type2.default.doubleClick) return new DoubleClickCommand(obj);

    if (obj.type === _type2.default.hover) return new HoverCommand(obj);

    if (obj.type === _type2.default.drag) return new DragCommand(obj);

    if (obj.type === _type2.default.dragToElement) return new DragToElementCommand(obj);

    if (obj.type === _type2.default.typeText) return new TypeTextCommand(obj);

    if (obj.type === _type2.default.testDone) return new TestDoneCommand();
}

function isTestDoneCommand(command) {
    return command.type === _type2.default.testDone;
}

function isCommandRejectableByPageError(command) {
    return !isTestDoneCommand(command) && command.type !== _type2.default.execHybridFn;
}