'use strict';

exports.__esModule = true;
exports.TypeOptions = exports.MoveOptions = exports.ClickOptions = exports.MouseOptions = exports.OffsetOptions = exports.SelectOptions = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _assignable = require('../../utils/assignable');

var _assignable2 = _interopRequireDefault(_assignable);

var _testRun = require('../../errors/test-run');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Validators
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

function integer(option, val, positive) {
    var valType = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);
    var ErrorCtor = positive ? _testRun.ActionPositiveIntegerOptionError : _testRun.ActionIntegerOptionError;

    if (valType !== 'number') throw new ErrorCtor(option, valType);

    var isInteger = !isNaN(val) && isFinite(val) && val === Math.floor(val);

    if (!isInteger) throw new ErrorCtor(option, val);
}

function positiveInteger(option, val) {
    integer(option, val, true);

    if (val < 0) throw new _testRun.ActionPositiveIntegerOptionError(option, val, true);
}

function positiveIntegerOrNull(option, val) {
    if (val === null) return;

    positiveInteger(option, val);
}

function boolean(option, val) {
    var valType = typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val);

    if (valType !== 'boolean') throw new _testRun.ActionBooleanOptionError(option, valType);
}

// Select

var SelectOptions = exports.SelectOptions = function (_Assignable) {
    (0, _inherits3.default)(SelectOptions, _Assignable);

    function SelectOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, SelectOptions);


        // NOTE: start and end positions can be defined by integer values (for 'inputs'
        // and 'textareas') or {node, offset} objects (for contentEditable elements)

        var _this = (0, _possibleConstructorReturn3.default)(this, _Assignable.call(this));

        _this.startPos = null; // TODO
        _this.endPos = null;

        _this._assignFrom(obj, validate);
        return _this;
    }

    SelectOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'startPos' }, { name: 'endPos' }];
    };

    return SelectOptions;
}(_assignable2.default);

// Offset


var OffsetOptions = exports.OffsetOptions = function (_Assignable2) {
    (0, _inherits3.default)(OffsetOptions, _Assignable2);

    function OffsetOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, OffsetOptions);

        var _this2 = (0, _possibleConstructorReturn3.default)(this, _Assignable2.call(this));

        _this2.offsetX = 0;
        _this2.offsetY = 0;

        _this2._assignFrom(obj, validate);
        return _this2;
    }

    OffsetOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return [{ name: 'offsetX', type: positiveInteger }, { name: 'offsetY', type: positiveInteger }];
    };

    return OffsetOptions;
}(_assignable2.default);

// Mouse


var MouseOptions = exports.MouseOptions = function (_OffsetOptions) {
    (0, _inherits3.default)(MouseOptions, _OffsetOptions);

    function MouseOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, MouseOptions);

        var _this3 = (0, _possibleConstructorReturn3.default)(this, _OffsetOptions.call(this));

        _this3.modifiers = {
            ctrl: false,
            alt: false,
            shift: false,
            meta: false
        };

        _this3._assignFrom(obj, validate);
        return _this3;
    }

    MouseOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return _OffsetOptions.prototype._getAssignableProperties.call(this).concat([{ name: 'modifiers.ctrl', type: boolean }, { name: 'modifiers.alt', type: boolean }, { name: 'modifiers.shift', type: boolean }, { name: 'modifiers.meta', type: boolean }]);
    };

    return MouseOptions;
}(OffsetOptions);

// Click


var ClickOptions = exports.ClickOptions = function (_MouseOptions) {
    (0, _inherits3.default)(ClickOptions, _MouseOptions);

    function ClickOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, ClickOptions);

        var _this4 = (0, _possibleConstructorReturn3.default)(this, _MouseOptions.call(this));

        _this4.caretPos = null;

        _this4._assignFrom(obj, validate);
        return _this4;
    }

    ClickOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return _MouseOptions.prototype._getAssignableProperties.call(this).concat([{ name: 'caretPos', type: positiveIntegerOrNull }]);
    };

    return ClickOptions;
}(MouseOptions);

// Move


var MoveOptions = exports.MoveOptions = function (_MouseOptions2) {
    (0, _inherits3.default)(MoveOptions, _MouseOptions2);

    function MoveOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, MoveOptions);

        var _this5 = (0, _possibleConstructorReturn3.default)(this, _MouseOptions2.call(this));

        _this5.speed = null;
        _this5.minMovingTime = null;
        _this5.dragMode = false;

        _this5._assignFrom(obj, validate);
        return _this5;
    }

    MoveOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return _MouseOptions2.prototype._getAssignableProperties.call(this).concat([{ name: 'speed' }, { name: 'minMovingTime' }, { name: 'dragMode' }]);
    };

    return MoveOptions;
}(MouseOptions);

// Type


var TypeOptions = exports.TypeOptions = function (_ClickOptions) {
    (0, _inherits3.default)(TypeOptions, _ClickOptions);

    function TypeOptions(obj, validate) {
        (0, _classCallCheck3.default)(this, TypeOptions);

        var _this6 = (0, _possibleConstructorReturn3.default)(this, _ClickOptions.call(this));

        _this6.replace = false;

        _this6._assignFrom(obj, validate);
        return _this6;
    }

    TypeOptions.prototype._getAssignableProperties = function _getAssignableProperties() {
        return _ClickOptions.prototype._getAssignableProperties.call(this).concat([{ name: 'replace', type: boolean }]);
    };

    return TypeOptions;
}(ClickOptions);