'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _readFileRelative = require('read-file-relative');

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _testcafeHammerhead = require('testcafe-hammerhead');

var _debugLog = require('./debug-log');

var _debugLog2 = _interopRequireDefault(_debugLog);

var _formattableAdapter = require('../errors/test-run/formattable-adapter');

var _formattableAdapter2 = _interopRequireDefault(_formattableAdapter);

var _commands = require('./commands');

var _clientMessages = require('./client-messages');

var _clientMessages2 = _interopRequireDefault(_clientMessages);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//Const
var TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../client/test-run/index.js.mustache');

var TestRun = function (_Session) {
    (0, _inherits3.default)(TestRun, _Session);

    function TestRun(test, browserConnection, screenshotCapturer, opts) {
        (0, _classCallCheck3.default)(this, TestRun);

        var uploadsRoot = _path2.default.dirname(test.fixture.path);

        var _this = (0, _possibleConstructorReturn3.default)(this, _Session.call(this, uploadsRoot));

        _this.opts = opts;
        _this.test = test;
        _this.browserConnection = browserConnection;
        _this.screenshotCapturer = screenshotCapturer;

        _this.running = false;
        _this.state = _state2.default.initial;

        _this.pendingCommand = null;
        _this.pendingRequest = null;
        _this.pendingJsError = null;
        _this.currentCommandCallsite = null;

        _this.debugLog = new _debugLog2.default(_this.browserConnection.userAgent);

        _this.injectable.scripts.push('/testcafe-core.js');
        _this.injectable.scripts.push('/testcafe-ui.js');
        _this.injectable.scripts.push('/testcafe-runner.js');
        _this.injectable.scripts.push('/testcafe-driver.js');
        _this.injectable.styles.push('/testcafe-ui-styles.css');

        _this.errs = [];
        return _this;
    }

    // Hammerhead payload


    TestRun.prototype._getPayloadScript = function _getPayloadScript() {
        return _mustache2.default.render(TEST_RUN_TEMPLATE, {
            testRunId: this.id,
            browserHeartbeatUrl: this.browserConnection.heartbeatUrl,
            browserStatusUrl: this.browserConnection.statusUrl,
            elementAvailabilityTimeout: this.opts.elementAvailabilityTimeout
        });
    };

    TestRun.prototype._getIframePayloadScript = function _getIframePayloadScript() {}
    // TODO


    // Hammerhead handlers
    ;

    TestRun.prototype.getAuthCredentials = function getAuthCredentials() {
        // TODO
    };

    TestRun.prototype.handleFileDownload = function handleFileDownload() {
        // TODO
    };

    TestRun.prototype.handlePageError = function handlePageError() {}
    // TODO


    // Test function execution
    ;

    TestRun.prototype._executeTestFn = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(state, fn) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            this.state = state;

                            _context.prev = 1;
                            _context.next = 4;
                            return fn(this);

                        case 4:
                            _context.next = 10;
                            break;

                        case 6:
                            _context.prev = 6;
                            _context.t0 = _context['catch'](1);

                            this._addError(_context.t0);
                            return _context.abrupt('return', false);

                        case 10:
                            return _context.abrupt('return', !this._addPendingErrorIfAny());

                        case 11:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this, [[1, 6]]);
        }));

        function _executeTestFn(_x, _x2) {
            return ref.apply(this, arguments);
        }

        return _executeTestFn;
    }();

    TestRun.prototype._start = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
            var beforeEachFn, afterEachFn;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            beforeEachFn = this.test.fixture.beforeEachFn;
                            afterEachFn = this.test.fixture.afterEachFn;


                            TestRun.activeTestRuns[this.id] = this;

                            this.running = true;
                            this.emit('start');

                            _context2.t0 = !beforeEachFn;

                            if (_context2.t0) {
                                _context2.next = 10;
                                break;
                            }

                            _context2.next = 9;
                            return this._executeTestFn(_state2.default.inBeforeEach, beforeEachFn);

                        case 9:
                            _context2.t0 = _context2.sent;

                        case 10:
                            if (!_context2.t0) {
                                _context2.next = 16;
                                break;
                            }

                            _context2.next = 13;
                            return this._executeTestFn(_state2.default.inTest, this.test.fn);

                        case 13:
                            if (!afterEachFn) {
                                _context2.next = 16;
                                break;
                            }

                            _context2.next = 16;
                            return this._executeTestFn(_state2.default.inAfterEach, afterEachFn);

                        case 16:
                            _context2.next = 18;
                            return this.executeCommand(new _commands.TestDoneCommand());

                        case 18:
                            this._addPendingErrorIfAny();

                            delete TestRun.activeTestRuns[this.id];

                            this.emit('done');

                        case 21:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _start() {
            return ref.apply(this, arguments);
        }

        return _start;
    }();

    // Errors


    TestRun.prototype._addPendingErrorIfAny = function _addPendingErrorIfAny() {
        if (this.pendingJsError) {
            this._addError(this.pendingJsError);
            this.pendingJsError = null;
            return true;
        }

        return false;
    };

    TestRun.prototype._addError = function _addError(err) {
        var adapter = new _formattableAdapter2.default(err, {
            userAgent: this.browserConnection.userAgent,
            screenshotPath: '',
            callsite: this.currentCommandCallsite,
            testRunState: this.state
        });

        this.errs.push(adapter);
    };

    // Pending command and request


    TestRun.prototype._resolvePendingCommand = function _resolvePendingCommand(commandResult) {
        this.pendingCommand.resolve(commandResult);
        this.pendingCommand = null;
    };

    TestRun.prototype._rejectPendingCommand = function _rejectPendingCommand(err) {
        this.pendingCommand.reject(err);
        this.pendingCommand = null;
    };

    TestRun.prototype._resolvePendingRequest = function _resolvePendingRequest(command) {
        this.pendingRequest.resolve(command);
        this.pendingRequest = null;
    };

    TestRun.prototype._addPendingCommand = function _addPendingCommand(command) {
        var _this2 = this;

        return new _pinkie2.default(function (resolve, reject) {
            return _this2.pendingCommand = { command: command, resolve: resolve, reject: reject };
        });
    };

    // Execute command


    TestRun.prototype.executeCommand = function executeCommand(command, callsite) {
        (0, _assert2.default)(!this.pendingCommand, 'Internal error: an attempt to execute a command when a previous command is still being executed was detected.');

        this.debugLog.command(command);

        this.currentCommandCallsite = callsite;

        if (this.pendingJsError && (0, _commands.isCommandRejectableByPageError)(command)) {
            var result = _pinkie2.default.reject(this.pendingJsError);

            this.pendingJsError = null;

            return result;
        }

        if (this.pendingRequest) this._resolvePendingRequest(command);

        return this._addPendingCommand(command);
    };

    return TestRun;
}(_testcafeHammerhead.Session);

// Active test runs pool, used by hybrid functions


exports.default = TestRun;
TestRun.activeTestRuns = {};

// Service message handlers
var ServiceMessages = TestRun.prototype;

ServiceMessages[_clientMessages2.default.ready] = function (msg) {
    var _this3 = this;

    var commandResult = msg.commandResult;

    this.debugLog.driverMessage(msg);

    this.pendingRequest = null;

    if (!this.running) this._start();

    if (this.pendingCommand) {
        //NOTE: ignore client messages if testDone command is received
        if (commandResult && !(0, _commands.isTestDoneCommand)(this.pendingCommand)) {
            if (commandResult.failed) this._rejectPendingCommand(commandResult.err);else this._resolvePendingCommand(commandResult);
        } else return _pinkie2.default.resolve(this.pendingCommand.command);
    }

    return new _pinkie2.default(function (resolve, reject) {
        return _this3.pendingRequest = { resolve: resolve, reject: reject };
    });
};

ServiceMessages[_clientMessages2.default.jsError] = function (msg) {
    this.debugLog.error(msg.err);

    if (this.pendingCommand && (0, _commands.isCommandRejectableByPageError)(this.pendingCommand)) this._rejectPendingCommand(msg.err);else this.pendingJsError = msg.err;
};

ServiceMessages[_clientMessages2.default.done] = function () {
    this.debugLog.testDone();
    this.pendingCommand.resolve();
};
module.exports = exports['default'];